@@ -61,9 +61,9 @@ import click
               help='Integrate the PDF over the bins instead of using the probability '
                    'density at the bin centre')
 @click.option('--parallel', type=int, default=-1, show_default=True,
-              help='\b Parallelize job across the N workers.'
-                   '\b Case  0: Jobs are run sequentially (for debugging).\n'
-                   '\b Case -1: Jobs are run across N_CPU workers.\n')
+              help='\b\n Parallelize job across the N workers.'
+                   '\b\n Case  0: Jobs are run sequentially (for debugging).'
+                   '\b\n Case -1: Jobs are run across N_CPU workers.')
 @click.option('--cache/--no-cache', default=True, show_default=True,
               help='Cache existing result')
 @click.option('--exclude', default="", show_default=True,
@@ -124,12 +124,12 @@ def run_pulls(**kwargs):
 @click.option('--lumi', default="139 fb$^{-1}$", show_default=True, 
               help='Luminosity')
 @click.option('--status', default="int", show_default=True, 
-              help='\b Analysis status. Choose from'
-                   '\b            int : Internal\n'
-                   '\b            wip : Work in Progress\n'
-                   '\b         prelim : Preliminary\n'
-                   '\b          final : *no status label*\n'
-                   '\b *custom input* : *custom input*\n')
+              help='\b\n Analysis status. Choose from'
+                   '\b\n            int : Internal'
+                   '\b\n            wip : Work in Progress'
+                   '\b\n         prelim : Preliminary'
+                   '\b\n          final : *no status label*'
+                   '\b\n *custom input* : *custom input*')
 @click.option('--combine_pdf/--split_pdf', default=True, show_default=True,
               help='Combine all ranking plots into a single pdf')
 @click.option('--outdir', default='ranking_plots', show_default=True,
@@ -149,78 +149,6 @@ def plot_pulls(**kwargs):
     inputdir, poi = kwargs.pop('inputdir'), kwargs.pop('poi')
     ranking_plot = NPRankingPlot(inputdir, poi)
     ranking_plot.plot(**kwargs)
-    
-    
-@click.command(name='likelihood_scan')
-@click.option('-i', '--input_file', 'filename', required=True, 
-              help='Path to the input workspace file.')
-@click.option('--min', 'poi_min', type=float, required=True, 
-              help='Minimum POI value to scan.')
-@click.option('--max', 'poi_max', type=float, required=True, 
-              help='Maximum POI value to scan.')
-@click.option('--step', 'poi_step', type=float, required=True, 
-              help='Scan interval.')
-@click.option('-p', '--poi', 'poi_name', default=None, show_default=True,
-              help='POI to scan. If not specified, the first POI from the workspace is used.')
-@click.option('--cache/--no-cache', default=True, show_default=True,
-              help='Cache existing result')
-@click.option('-o', '--outname', default='{poi_name}', show_default=True,
-              help='Name of output.')
-@click.option('--outdir', default='likelihood_scan', show_default=True,
-              help='Output directory.')
-@click.option('-w', '--workspace', 'ws_name', default=None, show_default=True,
-              help='Name of workspace. Auto-detect by default.')
-@click.option('-m', '--model_config', 'mc_name', default=None, show_default=True,
-              help='Name of model config. Auto-detect by default.')
-@click.option('-d', '--data', 'data_name', default='combData', show_default=True,
-              help='Name of dataset.')
-@click.option('-s', '--snapshot', 'snapshot_name', default=None, show_default=True,
-              help='Name of initial snapshot')
-@click.option('-r', '--profile', 'profile_param', default="", show_default=True,
-              help='Parameters to profile')
-@click.option('-f', '--fix', 'fix_param', default="", show_default=True,
-              help='Parameters to fix')
-@click.option('--constrain/--no-constrain', 'constrain_nuis', default=True, show_default=True,
-              help='Use constrained NLL (i.e. include systematics)')
-@click.option('-t', '--minimizer_type', default="Minuit2", show_default=True,
-              help='Minimizer type')
-@click.option('-a', '--minimizer_algo', default="Migrad", show_default=True,
-              help='Minimizer algorithm')
-@click.option('-c', '--num_cpu', type=int, default=1, show_default=True,
-              help='Number of CPUs to use per parameter')
-@click.option('--binned/--unbinned', 'binned_likelihood', default=True, show_default=True,
-              help='Binned likelihood')
-@click.option('-e', '--eps', type=float, default=1.0, show_default=True,
-              help='Convergence criterium')
-@click.option('--strategy', type=int, default=1, show_default=True,
-              help='Default minimization strategy')
-@click.option('--print_level', type=int, default=0, show_default=True,
-              help='Minimizer print level')
-@click.option('--fix-cache/--no-fix-cache', default=True, show_default=True,
-              help='Fix StarMomentMorph cache')
-@click.option('--fix-multi/--no-fix-cache',  default=True, show_default=True,
-              help='Fix MultiPdf level 2')
-@click.option('--max_calls', type=int, default=-1, show_default=True,
-              help='Maximum number of function calls')
-@click.option('--max_iters', type=int, default=-1, show_default=True,
-              help='Maximum number of Minuit iterations')
-@click.option('--optimize', type=int, default=2, show_default=True,
-              help='Optimize constant terms')
-@click.option('--offset/--no-offset', default=True, show_default=True,
-              help='Offset likelihood')
-@click.option('-v', '--verbosity', default='INFO', show_default=True,
-              type=click.Choice(["DEBUG", "INFO", "WARNING", "ERROR"], case_sensitive=False),
-              help='Verbosity level.')
-@click.option('--parallel', type=int, default=-1, show_default=True,
-              help='\b Parallelize job across the N workers.'
-                   '\b Case  0: Jobs are run sequentially (for debugging).\n'
-                   '\b Case -1: Jobs are run across N_CPU workers.\n')
-def likelihood_scan(**kwargs):
-    """
-    Tool for likelihood scan
-    """
-    from quickstats.components.likelihood import scan_nll
-    scan_nll(**kwargs)
 
 @click.command(name='cls_limit')
 @click.option('-i', '--input_file', 'filename', required=True, 
@@ -366,10 +294,10 @@ def harmonize_np(ws_files, reference, input_config_path, base_path, outfile):
 @click.option('-d', '--data', default='combData', show_default=True,
               help='Name of the dataset used in NP profiling.')
 @click.option('--constraint_option', default=0, show_default=True,
-              help='Customize the target of nuisance paramaters involved in the profiling.'
-                   'Case 0: All nuisance parameters are allowed to float;'
-                   'Case 1: Constrained nuisance parameters are fixed to 0. Unconstrained '
-                   'nuisrance parameters are allowed to float.')
+              help='\b\n Customize the target of nuisance paramaters involved in the profiling.'
+                   '\b\n Case 0: All nuisance parameters are allowed to float;'
+                   '\b\n Case 1: Constrained nuisance parameters are fixed to 0.'
+                   '\b\n         Unconstrained nuisrance parameters are allowed to float.')
 @click.option('-c', '--configuration', default=None,
               help='Path to the json configuration file containing'
                    ' the minimizer options for NP profiling.')
@@ -413,15 +341,15 @@ def generate_asimov(**kwargs):
 @click.option('-s', '--snapshot', 'snapshot_name', default=None, show_default=True,
               help='Name of initial snapshot')
 @click.option('-t', '--asimov_types', default="0,1,2", show_default=True,
-              help='\b Types of asimov dataset to generate separated by commas.\n'
-                   '\b 0: fit with POI fixed to 0\n'
-                   '\b 1: fit with POI fixed to 1\n'
-                   '\b 2: fit with POI free and set POI to 1 after fit\n'
-                   '\b 3: fit with POI and constrained NP fixed to 0\n'
-                   '\b 4: fit with POI fixed to 1 and constrained NP fixed to 0\n'
-                   '\b 5: fit with POI free and constrained NP fixed to 0 and set POI to 1 after fit\n'
-                   '\b -1: nominal NP with POI set to 0\n'
-                   '\b -2: nominal NP with POI set to 1\n')
+              help='\b\n Types of asimov dataset to generate separated by commas.'
+                   '\b\n 0: fit with POI fixed to 0'
+                   '\b\n 1: fit with POI fixed to 1'
+                   '\b\n 2: fit with POI free and set POI to 1 after fit'
+                   '\b\n 3: fit with POI and constrained NP fixed to 0'
+                   '\b\n 4: fit with POI fixed to 1 and constrained NP fixed to 0'
+                   '\b\n 5: fit with POI free and constrained NP fixed to 0 and set POI to 1 after fit'
+                   '\b\n -1: nominal NP with POI set to 0'
+                   '\b\n -2: nominal NP with POI set to 1')
 def generate_standard_asimov(**kwargs):
     """
     Generate standard Asimov dataset
@@ -470,9 +398,9 @@ def generate_standard_asimov(**kwargs):
               type=click.Choice(["DEBUG", "INFO", "WARNING", "ERROR"], case_sensitive=False),
               help='Verbosity level.')
 @click.option('--parallel', type=int, default=-1, show_default=True,
-              help='\b Parallelize job across the N workers.'
-                   '\b Case  0: Jobs are run sequentially (for debugging).\n'
-                   '\b Case -1: Jobs are run across N_CPU workers.\n')
+              help='\b\n Parallelize job across the N workers.'
+                   '\b\n Case  0: Jobs are run sequentially (for debugging).'
+                   '\b\n Case -1: Jobs are run across N_CPU workers.')
 def toy_significance(**kwargs):
     """
     Generate toys and evaluate significance
@@ -536,9 +464,9 @@ def toy_significance(**kwargs):
               help='Verbosity level.')
 @click.option('--snapshot', 'snapshot_name', default=None, help='Name of initial snapshot')
 @click.option('--parallel', type=int, default=-1, show_default=True,
-              help='\b Parallelize job across the N workers.'
-                   '\b Case  0: Jobs are run sequentially (for debugging).\n'
-                   '\b Case -1: Jobs are run across N_CPU workers.\n')
+              help='\b\n Parallelize job across the N workers.'
+                   '\b\n Case  0: Jobs are run sequentially (for debugging).'
+                   '\b\n Case -1: Jobs are run across N_CPU workers.')
 def toy_limit(**kwargs):
     """
     Generate toys and evaluate limits
