@@ -5,8 +5,8 @@ from quickstats.utils.common_utils import get_class_that_defined_method
 import ROOT
 
 class AnalysisObject(AbstractObject):
-    def __init__(self, filename:Optional[str]=None, poi_name:Optional[str]=None, 
-                 data_name:str='combData', binned_likelihood:bool=True, 
+    def __init__(self, filename:Optional[str]=None, poi_name:Optional[Union[str, List[str]]]=None,
+                 data_name:str='combData', binned_likelihood:bool=True,
                  fix_param:str='', profile_param:str='', ws_name:Optional[str]=None, 
                  mc_name:Optional[str]=None, snapshot_name:Optional[Union[List[str], str]]=None,
                  minimizer_type:str='Minuit2', minimizer_algo:str='Migrad', precision:float=0.001, 
@@ -18,16 +18,31 @@ class AnalysisObject(AbstractObject):
         super().__init__(verbosity=verbosity)    
         self.model = None
         self.minimizer = None
+        self._use_blind_range = False
         
         if filename is not None:
             self.setup_model(filename, data_name, binned_likelihood, ws_name, mc_name, 
                              snapshot_name, fix_cache, fix_multi, verbosity=verbosity)
+            self.save_snapshot("initialSnapshot")
             self.set_poi(poi_name)
-            self.setup_parameters(fix_param, profile_param)
+            self.setup_parameters(fix_param, profile_param, update_snapshot=True)
             self.setup_minimizer(minimizer_type, minimizer_algo, precision, eps, strategy, num_cpu,
                                  offset, optimize, eigen, max_calls, max_iters, print_level,
                                  timer, constrain_nuis, batch_mode, int_bin_precision,
                                  verbosity=verbosity)
+    
+    @property
+    def use_blind_range(self):
+        return self._use_blind_range
+    
+    @property
+    def minimizer_options(self):
+        return self.minimizer.config
+    
+    @property
+    def nll_commands(self):
+        return self.minimizer.nll_commands
+    
     # black magic
     def _inherit_init(self, init_func, **kwargs):
         import inspect
@@ -43,10 +58,22 @@ class AnalysisObject(AbstractObject):
             this_kwargs = {k:v for k,v in kwargs.items() if k not in that_parameters}
             init_func(config=this_kwargs, **that_kwargs)
 
-    def set_poi(self, poi_name:Optional[str]=None):
-        self.poi = self.model.get_poi(poi_name)
-        if poi_name is not None:
-            self.stdout.info('INFO: POI set to "{}"'.format(poi_name))            
+    def set_poi(self, poi_name:Optional[Union[str, List[str]]]=None):
+        # multi POI case
+        if isinstance(poi_name, list):
+            self.poi = ROOT.RooArgSet()
+            true_poi_names = []
+            # remove duplicates
+            poi_name = list(set(poi_name))
+            for pname in poi_name:
+                poi = self.model.get_poi(pname)
+                self.poi.add(poi)
+                true_poi_names.append(poi.GetName())
+            self.stdout.info('INFO: POIs set to {}'.format(", ".join([f"\"{name}\"" for name in true_poi_names])))
+        else:
+            self.poi = self.model.get_poi(poi_name)
+            if poi_name is not None:
+                self.stdout.info('INFO: POI set to "{}"'.format(poi_name))            
         
     def setup_model(self, filename, data_name='combData', binned_likelihood:bool=True, 
                     ws_name:Optional[str]=None, mc_name:Optional[str]=None, 
@@ -57,13 +84,15 @@ class AnalysisObject(AbstractObject):
                               fix_cache=fix_cache, fix_multi=fix_multi, verbosity=verbosity)
         self.model = model
     
-    def setup_parameters(self, fix_param:str='', profile_param:str=''):
+    def setup_parameters(self, fix_param:str='', profile_param:str='', update_snapshot:bool=True):
         if not self.model:
             raise RuntimeError('uninitialized analysis object')
         if fix_param:
             self.model.fix_parameters(fix_param)
         if profile_param:
             self.model.profile_parameters(profile_param)
+        if update_snapshot:
+            self.save_snapshot("currentSnapshot")
             
     def setup_minimizer(self, minimizer_type:str='Minuit2', minimizer_algo:str='Migrad', 
                         precision:float=0.001, eps:float=1.0, strategy:int=0, 
@@ -102,8 +131,6 @@ class AnalysisObject(AbstractObject):
         minimizer.configure(nll_commands=nll_commands, **minimizer_options)
         
         self.minimizer = minimizer
-        self.minimizer_options = minimizer_options
-        self.nll_commands      = nll_commands
     
     def set_data(self, data_name:str='combData'):
         data = self.model.workspace.data(data_name)
@@ -112,9 +139,40 @@ class AnalysisObject(AbstractObject):
         self.minimizer.set_data(data)
         self.model._data = data
         
+    def set_blind_range(self, blind_range:List[float], categories:Optional[List[str]]=None):
+        self.model.create_blind_range(blind_range, categories)
+        sideband_range_name = self.model.get_sideband_range_name()
+        self.minimizer.configure_nll(range=sideband_range_name, update=True)
+        self._use_blind_range = True
+        
+    def unset_blind_range(self):
+        self.minimizer.nll = None
+        self.minimizer.nll_commands.pop("RangeWithName", None)
+        self.stdout.info("INFO: Blind range removed from list of  NLL commands. NLL is reset.")
+        self._use_blind_range = False
+        
     def load_snapshot(self, snapshot_name:Optional[str]=None):
         if snapshot_name is not None:
-            self.model.workspace.loadSnapshot(snapshot_name)
+            snapshot = self.model.workspace.getSnapshot(snapshot_name)
+            if not snapshot:
+                self.stdout.warning(f"WARNING: Snapshot \"{snapshot_name}\" does not exist.")
+            else:
+                self.model.workspace.loadSnapshot(snapshot_name)
+                self.stdout.debug(f"DEBUG: Loaded snapshot \"{snapshot_name}\"")
+                
+    def save_snapshot(self, snapshot_name:Optional[str]=None, 
+                      variables:Optional[Union[ROOT.RooArgSet, str]]=None):
+        if snapshot_name is not None:
+            if variables is None:
+                self.model.workspace.saveSnapshot(snapshot_name, self.model.workspace.allVars())
+            else:
+                if isinstance(variables, str):
+                    if variables == "nuis":
+                        variables = self.model.nuisance_parameters
+                    elif variables == "globs":
+                        variables = self.model.global_observables
+                self.model.workspace.saveSnapshot(snapshot_name, variables)
+            self.stdout.debug(f"DEBUG: Saved snapshot \"{snapshot_name}\"")
         
     def save(self, fname:str, recreate:bool=True):
         self.model.save(fname, recreate=recreate)
