@@ -80,7 +80,8 @@ class ExtendedMinimizer(AbstractObject):
         
         self.fit_options = {}
         self.scan_options = {}
-        self.nll_command_list = ROOT.RooLinkedList()
+        #self.nll_command_list = ROOT.RooLinkedList()
+        self.nll_commands = {}
         
         self.configure_default_minimizer_options()
         
@@ -131,14 +132,24 @@ class ExtendedMinimizer(AbstractObject):
     def set_data(self, data:"ROOT.RooAbsData"):
         self.data = data
     
-    def create_nll(self, nll_command_list:List["ROOT.RooCmdArg"]=None):
+    def _create_nll(self, nll_commands:List["ROOT.RooCmdArg"]=None, 
+                    dataset:Optional["ROOT.RooDataSet"]=None):
+        if nll_commands is None:
+            nll_commands = list(self.nll_commands.values())
+        #self.nll = self.pdf.createNLL(self.data, nll_command_list)
+        for command in nll_commands:
+            if command.GetName() == "RangeWithName":
+                range_name = command.getString(0)
+                self.stdout.info(f"INFO: Using the range \"{range_name}\" for NLL calculation")
+        if dataset is None:
+            dataset = self.data
+        nll = self.pdf.createNLL(dataset, *nll_commands)
+        return nll
+            
+    def create_nll(self, nll_commands:List["ROOT.RooCmdArg"]=None, 
+                   dataset:Optional["ROOT.RooDataSet"]=None):
         if (not self.nll) or (not self.config['reuse_nll']):
-            if nll_command_list is None:
-                nll_command_list = self.nll_command_list
-            else:
-                nll_command_list = self.parse_nll_commands(nll_command_list)
-            #self.nll = self.pdf.createNLL(self.data, nll_command_list)
-            self.nll = self.pdf.createNLL(self.data, *nll_command_list)
+            self.nll = self._create_nll(nll_commands, dataset)
         
     def create_minimizer(self):
         
@@ -162,21 +173,15 @@ class ExtendedMinimizer(AbstractObject):
                 self.minimizer.setMaxFunctionCalls(self.config['max_calls'])
             if self.config['max_iters'] != -1:
                 self.minimizer.setMaxIterations(self.config['max_iters'])
- 
-    def parse_nll_commands(self, commands:List["ROOT.RooCmdArg"]):
-        # temporary fix for ROOT version below 6.22/06
-        #nll_command_list = ROOT.RooLinkedList()
-        #for command in commands:
-        #    nll_command_list.Add(command)
-        #return nll_command_list
-        return commands
-        
-    def configure_nll(self, constrain=None, global_observables=None, **kwargs):
+    
+    def construct_nll_commands(self, constrain=None, global_observables=None, **kwargs):
         commands = []
         if 'num_cpu' in kwargs:
             commands.append(ROOT.RooFit.NumCPU(kwargs['num_cpu'], 3))
         if 'offset' in kwargs:
             commands.append(ROOT.RooFit.Offset(kwargs['offset']))
+        if 'range' in kwargs:
+            commands.append(ROOT.RooFit.Range(kwargs['range']))
         if ('batch_mode' in kwargs) and hasattr(ROOT.RooFit, "BatchMode"):
             commands.append(ROOT.RooFit.BatchMode(batch_mode))
         if ('int_bin_precision' in kwargs) and hasattr(ROOT.RooFit, "IntegrateBins"):
@@ -185,8 +190,26 @@ class ExtendedMinimizer(AbstractObject):
             commands.append(ROOT.RooFit.Constrain(constrain))
             self.stdout.warning("Use of `Constrain` option from createNLL is not recommended")
         if global_observables is not None:
-            commands.append(ROOT.RooFit.GlobalObservables(global_observables))            
-        self.nll_command_list = self.parse_nll_commands(commands)
+            commands.append(ROOT.RooFit.GlobalObservables(global_observables))
+        command_dict = {}
+        for command in commands:
+            command_dict[command.GetName()] = command
+        # temporary fix for ROOT version below 6.22/06
+        #nll_command_list = ROOT.RooLinkedList()
+        #for command in commands:
+        #    nll_command_list.Add(command)
+        #return nll_command_list
+        return command_dict
+        
+    def configure_nll(self, constrain=None, global_observables=None, update:bool=False, **kwargs):
+        command_dict = self.construct_nll_commands(constrain=constrain, 
+                                                   global_observables=global_observables,
+                                                   **kwargs)
+        if update:
+            self.nll_commands.update(command_dict)
+        else:
+            self.nll_commands = command_dict
+        self.nll = None
     
     def configure(self, **kwargs):
         minos_set = kwargs.pop('minos_set', None)
@@ -201,7 +224,7 @@ class ExtendedMinimizer(AbstractObject):
         # parse nll commands 
         nll_commands = kwargs.pop('nll_commands', [])
         if nll_commands:
-            self.nll_command_list = self.parse_nll_commands(nll_commands)
+            self.nll_commands = {cmd.GetName():cmd for cmd in nll_commands}
         
         # configure minimizer settings
         for arg in kwargs:
