@@ -7,7 +7,6 @@ import os
 import sys
 import copy
 import math
-import logging
 import fnmatch
 from typing import List, Optional, Union, Dict, Set
 
@@ -19,10 +18,11 @@ import ROOT
 import quickstats
 from quickstats import semistaticmethod
 from quickstats.components import AbstractObject
-from quickstats.components.numerics import is_integer, pretty_value
+from quickstats.maths.numerics import is_integer, pretty_value
 from quickstats.utils.root_utils import load_macro
 from quickstats.utils.common_utils import str_list_filter
 from quickstats.utils.roofit_utils import get_variable_attributes, variable_collection_to_dataframe
+from quickstats.interface.root import TH1, RooDataSet
 
 class ExtendedModel(AbstractObject):
     
@@ -36,7 +36,10 @@ class ExtendedModel(AbstractObject):
         'dataset_args': 'obsAndWeight',
         'asimov': 'asimovData_{mu}',
         'channel_asimov': 'combAsimovData_{label}',
-        'nll_snapshot': '{nll_name}_{mu}'
+        'nll_snapshot': '{nll_name}_{mu}',
+        'range_sideband_low': 'SBLo',
+        'range_blind': 'Blind',
+        'range_sideband_high': 'SBHi'
     }
     
     def __init__(self, fname:str, ws_name:Optional[str]=None, mc_name:Optional[str]=None,
@@ -300,7 +303,7 @@ class ExtendedModel(AbstractObject):
         '''
             source: parameters instance
             param_expr: 
-        '''            
+        '''
         param_dict = self.parse_param_expr(param_expr)
         return self._set_parameters(source, param_dict, mode='fix', param_str=param_str)           
     
@@ -444,19 +447,8 @@ class ExtendedModel(AbstractObject):
             # case both parameter name and value is given
             elif len(expr) == 2:
                 param_name = expr[0]
-                param_value = expr[1]
-                # range like expression
-                if ':' in param_value:
-                    param_range = param_value.split(':')
-                    if len(param_range) != 2:
-                        raise ValueError('invalid parameter range: {}'.format(param_value))
-                    param_min = float(param_range[0]) if param_range[0].isnumeric() else None
-                    param_max = float(param_range[1]) if param_range[1].isnumeric() else None
-                    param_dict[param_name] = [param_min, param_max]
-                elif is_integer(param_value):
-                    param_dict[param_name] = int(param_value)
-                else:
-                    param_dict[param_name] = float(param_value)
+                param_values = [float(v) for v in expr[1].split("_")]
+                param_dict[param_name] = param_values
             else:
                 raise ValueError('invalid parameter expression: {}'.format(param))
         return param_dict
@@ -604,10 +596,14 @@ class ExtendedModel(AbstractObject):
         return nuis_list, glob_list, pdf_list
     
     def get_constrained_nuisance_parameters(self):
+        """Get the list of constrained nuisance parameters instances
+        """
         constrained_nuis, _, _ = self.pair_constraints()
         return constrained_nuis
     
     def get_unconstrained_nuisance_parameters(self, constrained_nuis=None):
+        """Get the list of unconstrained nuisance parameters instances
+        """
         # do not re-isolate constrained nuisance parameters
         if constrained_nuis is None:
             constrained_nuis = self.get_constrained_nuisance_parameters()
@@ -699,7 +695,98 @@ class ExtendedModel(AbstractObject):
                             nuip_nom = 1.0
                             self.stdout.info("INFO: Assume that {} is nominal value of the nuisance parameter".format(nuip_nom))
         return prefit_variation, constraint_type, nuip_nom
+    
+    def create_blind_range(self, blind_range:List[float], categories:Optional[List[str]]=None):
+        """Create blind ranges for observables
         
+        Arguments
+            blind_range: list of float
+                A list of the form [<lower_blind_range>, <upper_blind_range>]
+            categories: (Optional) list of string
+                List of categories from which the observables will have their blind range created.
+                All categories are used by default.
+        """
+        if len(blind_range) != 2:
+            raise ValueError("invalid format for blind range, must be list of the form [xmin, xmax]")
+        range_name_SBLo = self._DEFAULT_NAMES_['range_sideband_low']
+        range_name_Blind = self._DEFAULT_NAMES_['range_blind']
+        range_name_SBHi = self._DEFAULT_NAMES_['range_sideband_high']
+        padding = max([len(range_name_SBLo), len(range_name_Blind), len(range_name_SBHi)]) + 2
+        blind_min = blind_range[0]
+        blind_max = blind_range[1]
+        category_map = self.get_category_map()
+        if categories is None:
+            categories = list(category_map)
+        for category in categories:
+            obs_name = category_map[category]['observable']
+            obs = self.workspace.var(obs_name)
+            obs_range = obs.getRange()
+            obs_min = obs_range.first
+            obs_max = obs_range.second
+            if (blind_max <= blind_min) or (blind_max > obs_max) or (blind_min < obs_min):
+                raise ValueError(f"invalid blinding range provided: min={blind_min}, max={blind_max}")
+            obs.setRange(f"{range_name_SBLo}", obs_min, blind_min)
+            obs.setRange(f"{range_name_Blind}", blind_min, blind_max)
+            obs.setRange(f"{range_name_SBHi}", blind_max, obs_max)
+            self.stdout.info(f"INFO: The following ranges are defined for the observable \"{obs_name}\"")
+            self.stdout.info(f"\t{range_name_SBLo.ljust(padding)}: [{obs_min}, {blind_min}]")
+            self.stdout.info(f"\t{range_name_Blind.ljust(padding)}: [{blind_min}, {blind_max}]")
+            self.stdout.info(f"\t{range_name_SBHi.ljust(padding)}: [{blind_max}, {obs_max}]")
+        return None
+    
+    def get_sideband_range_name(self):
+        """Get sideband range name to be used in RooAbsPdf.createNLL(..., Range(<range_name>))
+        """
+        range_name_SBLo = self._DEFAULT_NAMES_['range_sideband_low']
+        range_name_SBHi = self._DEFAULT_NAMES_['range_sideband_high']
+        return f"{range_name_SBLo},{range_name_SBHi}"
+    
+    def get_blind_range_name(self):
+        """Get blind range name that are excluded from the fit range of the observable
+        """
+        return self._DEFAULT_NAMES_['range_blind']
+    
+    def get_blind_range(self):
+        category_map = self.get_category_map()
+        blind_range_name = self.get_blind_range_name()
+        blind_range = {}
+        for category in category_map:
+            obs_name = category_map[category]['observable']
+            obs = self.workspace.var(obs_name)
+            dummy_range = obs.getRange(blind_range_name)
+            dummy_min = dummy_range.first
+            dummy_max = dummy_range.second
+            obs_range = obs.getRange()
+            obs_min = obs_range.first
+            obs_max = obs_range.second
+            if (dummy_min == obs_min) and (dummy_max == obs_max):
+                # observable not blinded, skipping
+                continue
+            blind_range[obs_name] = [dummy_min, dummy_max]
+        return blind_range
+    
+    def remove_blind_range(self, categories:Optional[List[str]]=None):
+        category_map = self.get_category_map()
+        if categories is None:
+            categories = list(category_map)
+        range_name_SBLo = self._DEFAULT_NAMES_['range_sideband_low']
+        range_name_Blind = self._DEFAULT_NAMES_['range_blind']
+        range_name_SBHi = self._DEFAULT_NAMES_['range_sideband_high']
+        blind_range = self.get_blind_range()
+        for category in categories:
+            obs_name = category_map[category]['observable']
+            if obs_name not in blind_range:
+                continue
+            obs = self.workspace.var(obs_name)
+            obs_range = obs.getRange()
+            obs_min = obs_range.first
+            obs_max = obs_range.second            
+            obs.setRange(range_name_SBLo, obs_min, obs_max)
+            obs.setRange(range_name_Blind, obs_min, obs_max)
+            obs.setRange(range_name_SBHi, obs_min, obs_max)
+            self.stdout.info(f"INFO: The following ranges are removed from the observable \"{obs_name}\":")
+            self.stdout.info(f"\t{range_name_SBLo}, {range_name_Blind}, {range_name_SBHi}")
+    
     def set_initial_errors(self, source:Optional["ROOT.RooArgSet"]=None):
         if not source:
             source = self.nuisance_parameters
@@ -715,6 +802,14 @@ class ExtendedModel(AbstractObject):
         return None
     
     def get_poi(self, poi_name:Optional[str]=None, strict:bool=False):
+        """Get POI variable by name
+        
+        Arguments
+            poi_name: str
+                name of POI to retrieve
+            strict: bool
+                require the variable to be defined in the POI list, throw error otherwise
+        """
         if poi_name is None:
             poi = self.pois.first()
             self.stdout.info(f"INFO: POI name not specified. The first POI \"{poi.GetName()}\" is used by default.")
@@ -1171,16 +1266,19 @@ class ExtendedModel(AbstractObject):
             category_map[label]['index'] = i
             category_map[label]['pdf'] = pdf_cat.GetName()
             category_map[label]['observable'] = target_obs.GetName()
+            bin_range = target_obs.getRange()
+            category_map[label]['bin_range'] = (bin_range.first, bin_range.second)
+            category_map[label]['bins'] = target_obs.getBins()
         return category_map
     
     @staticmethod
     def get_dataset_values(dataset):
         parameters = dataset.get()
-        observables = []
+        observables = ROOT.RooArgSet()
         category = None
         for p in parameters:
             if isinstance(p, ROOT.RooRealVar):
-                observables.append(p)
+                observables.add(p)
             elif isinstance(p, ROOT.RooCategory):
                 if category is None:
                     category = p
@@ -1188,17 +1286,16 @@ class ExtendedModel(AbstractObject):
                     raise RuntimeError("multiple RooCategory instances found in the dataset")
             else:
                 raise RuntimeError(f"unknown object type \"{type(p)}\" found in the dataset")
+        """
         n_entries = dataset.numEntries()
         result = {}
-        """
         # only available after ROOT 6.24
-        data_buffer = ROOT.RooBatchCompute.RunContext()
-        dataset.getBatches(data_buffer, 0, n_entries)
-        for obs in observables:
-            obs_name = obs.GetName()
-            data = np.asarray(data_buffer.getBatch(obs))
-            result[obs_name] = data
-        """
+        #data_buffer = ROOT.RooBatchCompute.RunContext()
+        #dataset.getBatches(data_buffer, 0, n_entries)
+        #for obs in observables:
+        #    obs_name = obs.GetName()
+        #    data = np.asarray(data_buffer.getBatch(obs))
+        #    result[obs_name] = data
         for obs in observables:
             obs_name = obs.GetName()
             result[obs_name] = np.empty(n_entries)
@@ -1213,14 +1310,39 @@ class ExtendedModel(AbstractObject):
                 dataset.get(i)
                 result['label'][i] = category.getLabel()
                 result['index'][i] = category.getIndex()
+        """
+        result = RooDataSet.extract_category_data(dataset, observables, category)
         return result
     
-    def get_dataset_distributions(self, dataset):
+    def _get_new_binnings(self, bins:Optional[Union[Dict, int]]=None):
+        category_map = self.get_category_map()
+        binnings = {}
+        for category in category_map:
+            binnings[category] = {}
+            bin_range = category_map[category]['bin_range']
+            if bins is None:
+                bins = category_map[category]['bins']
+            elif isinstance(bins, dict):
+                bins = bins.get(category, None)
+                if bins is None:
+                    raise RuntimeError(f"binning not specified for the category \"{category}\"")
+            elif isinstance(bins, int):
+                bins = bins
+            else:
+                raise RuntimeError(f"invalid binning format: {bins}")
+            bin_width = round((bin_range[1] - bin_range[0]) / bins, 8)
+            binnings[category]['bin_range'] = bin_range
+            binnings[category]['bins'] = bins
+            binnings[category]['bin_width'] = bin_width
+        return binnings
+    
+    def get_dataset_distributions(self, dataset, n_bins:Optional[Union[Dict, int]]=None):
         distributions = {}
         dataset_values = self.get_dataset_values(dataset)
         if 'label' not in dataset_values:
             raise RuntimeError("no categories defined in the dataset")
         category_map = self.get_category_map()
+        binnings = self._get_new_binnings(n_bins)
         for cat in category_map:
             distributions[cat] = {}
             observable = category_map[cat]['observable']
@@ -1232,27 +1354,59 @@ class ExtendedModel(AbstractObject):
             ind = np.argsort(x)
             x = x[ind]
             y = dataset_values['weight'][mask][ind]
-            distributions[cat]['x'] = x
-            distributions[cat]['y'] = y
+            default_bins = category_map[cat]['bins']
+            if np.all(y == 1.):
+                binned = False
+            elif len(x) == default_bins:
+                binned = True
+            else:
+                raise RuntimeError(f"detected dataset with invalid binning")
+            bins = binnings[cat]['bins']
+            if binned:
+                if  (bins != default_bins):
+                    raise RuntimeError(f"cannot change binning of binned dataset ({dataset.GetName()}, cat = {cat}): "
+                                       f"original binning = {default_bins}, requested binning = {bins}")
+                distributions[cat]['x'] = x
+                distributions[cat]['y'] = y
+            else:
+                bin_range = binnings[cat]['bin_range']
+                hist, bin_edges = np.histogram(x, bins=bins, range=(bin_range[0], bin_range[1]))
+                bin_centers = (bin_edges[1:] + bin_edges[:-1])/2
+                distributions[cat]['x'] = bin_centers
+                distributions[cat]['y'] = hist
+                bin_errors = TH1.GetPoissonError(hist)
+                distributions[cat]['yerrlo'] = bin_errors['lo']
+                distributions[cat]['yerrhi'] = bin_errors['hi']
         return distributions
     
     @semistaticmethod
-    def get_simul_pdf_distributions(self, pdf, observables):
+    def get_simul_pdf_distributions(self, pdf, observables, n_bins:Optional[Union[Dict, int]]=None):
         if not isinstance(pdf, ROOT.RooSimultaneous):
             raise ValueError("input pdf is not a simultaneous pdf")
         distributions = {}
         channel_cat = pdf.indexCat()
         n_cat = len(channel_cat)
+        binnings = self._get_new_binnings(n_bins)
         for i in range(n_cat):
             channel_cat.setIndex(i)
             label = channel_cat.getLabel()
             pdf_cat = pdf.getPdf(label)
             obs = pdf_cat.getObservables(observables)
             target_obs = obs.first()
-            expected_events = pdf_cat.expectedEvents(obs)
-            n_bins = target_obs.numBins()
             obs_name = target_obs.GetName()
             distributions[label] = {'x':[], 'y':[], 'observable': obs_name}
+            if n_bins is None:
+                n_bins = target_obs.numBins()
+            bins = binnings[label]['bins']
+            bin_width = binnings[label]['bin_width']
+            h = pdf_cat.createHistogram(target_obs.GetName(), bins)
+            py_h = TH1(h)
+            distributions[label]['x'] = py_h.bin_center
+            distributions[label]['y'] = py_h.bin_content * bin_width
+            # free memory to avoid memory leak
+            h.Delete()
+            """
+            expected_events = pdf_cat.expectedEvents(obs)
             for i in range(n_bins):
                 target_obs.setBin(i)
                 norm = pdf_cat.getVal(obs)*target_obs.getBinWidth(i)
@@ -1267,23 +1421,20 @@ class ExtendedModel(AbstractObject):
                     distributions[label]['y'].append(n_events)
                 else:
                     raise RuntimeError(f"detected pdf bin with nan (pdf={pdf.GetName()},obs={target_obs.GetName()},bin={i})")
+            """
         return distributions
     
-    def get_distributions(self):
-        return self.get_simul_pdf_distributions(self.pdf, self.observables)
+    def get_distributions(self, n_bins:Optional[Union[Dict, int]]=None):
+        return self.get_simul_pdf_distributions(self.pdf, self.observables, n_bins=n_bins)
     
     def get_categories(self):
         return list(self.get_category_map())
     
     def get_collected_distributions(self, current_distributions=True,
                                     datasets:Optional[Union[List[str],List[ROOT.RooDataSet]]]=None,
-                                    snapshots:Optional[List[str]]=None):
+                                    snapshots:Optional[List[str]]=None,
+                                    n_bins:Optional[Union[Dict, int]]=None):
         collected_distributions = {}
-        
-        if current_distributions:
-            distributions = self.get_distributions()
-            for category in distributions:
-                collected_distributions[category] = {"Current": distributions[category]}
                 
         if datasets is not None:
             for dataset in datasets:
@@ -1293,11 +1444,19 @@ class ExtendedModel(AbstractObject):
                     if not dataset:
                         raise ValueError(f"dataset \"{dataset_name}\" does not exist")
                 dataset_name = dataset.GetName()
-                distributions = self.get_dataset_distributions(dataset)
+                distributions = self.get_dataset_distributions(dataset, n_bins=n_bins)
                 for category in distributions:
                     if category not in collected_distributions:
                         collected_distributions[category] = {}
                     collected_distributions[category][dataset_name] = distributions[category]
+
+        if current_distributions:
+            distributions = self.get_distributions(n_bins=n_bins)
+            for category in distributions:
+                if category not in collected_distributions:
+                    collected_distributions[category] = {}
+                collected_distributions[category]['Current'] = distributions[category]
+
         if snapshots is not None:
             self.workspace.saveSnapshot("tmp", self.workspace.allVars())
             try:
@@ -1305,7 +1464,7 @@ class ExtendedModel(AbstractObject):
                     exist = self.workspace.loadSnapshot(snapshot)
                     if not exist:
                         raise RuntimeError("snapshot \"{}\" not found in workspace".format(snapshot))
-                    distributions = self.get_distributions()
+                    distributions = self.get_distributions(n_bins=n_bins)
                     for category in distributions:
                         if category not in collected_distributions:
                             collected_distributions[category] = {}
@@ -1314,18 +1473,60 @@ class ExtendedModel(AbstractObject):
                 self.workspace.loadSnapshot("tmp")
         return collected_distributions
             
-    def plot_distributions(self, category:str, current_distributions=True, 
+    def plot_distributions(self, categories:Optional[List[str]]=None, current_distributions=True, 
                            datasets:Optional[Union[List[str],List[ROOT.RooDataSet]]]=None,
-                           snapshots:Optional[Union[List[str], List[ROOT.RooArgSet]]]=None):
-        from quickstats.plots.template import plot_distributions
+                           snapshots:Optional[Union[List[str], List[ROOT.RooArgSet]]]=None,
+                           n_bins:Optional[Union[Dict, int]]=None, discriminant:Optional[str]=None,
+                           unit:Optional[str]=None, blind:bool=False, show_errorbar:bool=True,
+                           comparison_options:Optional[Dict]=None, save_as:Optional[str]=None, **kwargs):
+        from quickstats.plots import PdfDistributionPlot
         category_map = self.get_category_map()
-        if category not in category_map:
-            raise ValueError("category \"{}\" not found in workspace".format(category))
+        if categories is None:
+            categories = list(category_map)
+        for category in categories:
+            if category not in category_map:
+                raise ValueError("category \"{}\" not found in workspace".format(category))
         collected_distributions = self.get_collected_distributions(current_distributions,
-                                                                   datasets, snapshots=snapshots)
-        xlabel = category_map[category]['observable']
-        ylabel = "Events"
-        return plot_distributions(collected_distributions[category], xlabel=xlabel, ylabel=ylabel)
+                                                                   datasets, snapshots=snapshots,
+                                                                   n_bins=n_bins)
+        if save_as is not None:
+            from matplotlib.backends.backend_pdf import PdfPages
+            pdf = PdfPages(save_as)
+        else:
+            pdf = None
+            
+        if blind:
+            all_blind_range = self.get_blind_range()
+        else:
+            all_blind_range = {}
+
+        for i, category in enumerate(categories):
+            obs_name = category_map[category]['observable']
+            if discriminant is None:
+                xlabel = obs_name
+            else:
+                #xlabel = f"{discriminant}_{category}"
+                xlabel = f"{discriminant} (category {category})"
+            binnings  = self._get_new_binnings(n_bins)
+            bin_width = pretty_value(binnings[category]['bin_width'])
+            ylabel    = f"Events / {bin_width}"
+            if unit is not None:
+                xlabel = f"{xlabel} [{unit}]"
+                ylabel = f"{ylabel} {unit}"
+                
+            blind_range = all_blind_range.get(obs_name, None)
+            
+            plotter = PdfDistributionPlot(collected_distributions[category], 
+                                          figure_index=i, **kwargs)
+            ax = plotter.draw(xlabel=xlabel, ylabel=ylabel, 
+                              blind_range=blind_range,
+                              show_errorbar=show_errorbar,
+                              comparison_options=comparison_options)
+            if pdf is not None:
+                pdf.savefig(bbox_inches="tight")
+        if pdf is not None:
+            pdf.close()
+        return None
     
     def as_dataframe(self, variable_type:str):
         if variable_type == "pois":
@@ -1366,17 +1567,27 @@ class ExtendedModel(AbstractObject):
         # categories
         if 'category' in items:
             if isinstance(self.pdf, ROOT.RooSimultaneous):
-                channel_cat = self.pdf.indexCat()
-                n_cat = len(channel_cat)
+                category_map = self.get_category_map()
+                n_cat = len(category_map)
                 summary_str += f"Categories ({n_cat}):\n"
-                for i in range(n_cat):
-                    channel_cat.setIndex(i)
-                    label = channel_cat.getLabel()
-                    summary_str += f"\t{label}\n"
+                for category in category_map:
+                    observable = category_map[category]['observable']
+                    bin_range = category_map[category]['bin_range']
+                    bins = category_map[category]['bins']
+                    summary_str += f"\t{category} (observable = {observable}, range = [{bin_range[0]}, {bin_range[1]}], bins = {bins})\n"
+        """
         # observables
         if 'observable' in items:
             summary_str += f"Observables ({len(self.observables)}):\n"
-            summary_str += "".join([f"\t{obs.GetName()}\n" for obs in self.observables])
+            for obs in self.observables:
+                if obs.ClassName() != "RooRealVar":
+                    continue
+                obs_name = obs.GetName()
+                bin_range = obs.getRange()
+                n_bin = obs.getBins()
+                obs_str = f"{obs_name} (range = [{bin_range.first}, {bin_range.second}], bins = {n_bin})"
+                summary_str += f"\t{obs_str}\n"
+        """
         # pois, NPs
         param_strs     = []
         param_sets     = []
