@@ -9,6 +9,9 @@ import pandas as pd
 
 import ROOT
 
+from quickstats.utils.root_utils import declare_expression
+from quickstats.utils.common_utils import is_valid_file
+
 class RooProcBaseAction(object):
     def __init__(self, **params):
         self._params = params
@@ -171,7 +174,31 @@ class RooProcAlias(RooProcRDFAction):
         alias = params['alias']
         column_name = params['column_name']
         rdf_next = rdf.Alias(alias, column_name)
-        return rdf_next    
+        return rdf_next
+    
+class RooProcSafeAlias(RooProcHybridAction):
+    
+    def __init__(self, alias:str, column_name:str):
+        super().__init__(alias=alias, column_name=column_name)
+        
+    @classmethod
+    def parse(cls, text):
+        result = re.search(r"^\s*(\w+)\s*=\s*([\w\.]+)", text)
+        if not result:
+            raise RuntimeError(f"invalid expression {text}")
+        alias = result.group(1)
+        column_name = result.group(2)
+        return cls(alias=alias, column_name=column_name)    
+    
+    def _execute(self, rdf, processor, **params):
+        alias = params['alias']
+        column_name = params['column_name']
+        all_column_names = [str(i) for i in rdf.GetColumnNames()]
+        if column_name not in all_column_names:
+            processor.stdout.warning(f"WARNING: Column name `{column_name}` does not exist. No alias made.")
+            return rdf, processor
+        rdf_next = rdf.Alias(alias, column_name)
+        return rdf_next, processor
     
 class RooProcDefine(RooProcRDFAction):
     
@@ -238,6 +265,9 @@ class RooProcSave(RooProcHybridAction):
     def _execute(self, rdf, processor, **params):
         treename = params['treename']
         filename = params['filename']
+        if processor.cache and is_valid_file(filename):
+            processor.stdout.info(f"INFO: Cached output `{filename}`.")
+            return rdf, processor
         columns = params.get('columns', None)
         if columns is None:
             rdf_next = rdf.Snapshot(treename, filename)
@@ -252,7 +282,7 @@ class RooProcSave(RooProcHybridAction):
         processor.stdout.info(f"INFO: Writing output to `{filename}`.")
         return rdf_next, processor
     
-class RooProcAsNumpy(RooProcRDFAction):
+class RooProcAsNumpy(RooProcHybridAction):
     
     def __init__(self, filename:str, 
                  columns:Optional[List[str]]=None):
@@ -264,15 +294,18 @@ class RooProcAsNumpy(RooProcRDFAction):
         kwargs = cls.parse_as_kwargs(text)
         return cls(**kwargs)
     
-    def _execute(self, rdf, **kwargs):
+    def _execute(self, rdf, processor, **params):
         filename = kwargs['filename']
+        if processor.cache and is_valid_file(filename):
+            processor.stdout.info(f"INFO: Cached output `{filename}`.")
+            return rdf, processor
         columns = kwargs.get('columns', None)
         if columns is None:
             data = rdf.AsNumpy()
         else:
             data = rdf.AsNumpy(columns)
         np.save(filename, data)
-        return rdf    
+        return rdf, processor
     
 class RooProcReport(RooProcHybridAction):
     def __init__(self, display:bool=False, filename:Optional[str]=None):
@@ -286,6 +319,9 @@ class RooProcReport(RooProcHybridAction):
     def _execute(self, rdf, processor, **params):
         display = params['display']
         filename = params['filename']
+        if processor.cache and is_valid_file(filename):
+            processor.stdout.info(f"INFO: Cached output `{filename}`.")
+            return rdf, processor        
         cut_report = rdf.Report()
         result = []
         cumulative_eff  = 1
@@ -354,29 +390,11 @@ class RooProcDeclare(RooProcHelperAction):
     @classmethod
     def parse(cls, text):
         return cls(expression=text)        
-        
-    @staticmethod
-    def create_declaration(expression:str, name:Optional[str]=None):
-        if name is None:
-            hash_str = str(hash(expression)).replace("-", "n")
-            name_guard = f"__RooProcDeclare_{hash_str}__"
-        else:
-            name_guard = f"__RooProcDeclare_{name}__"
-        guarded_declaration = f"#ifndef {name_guard}\n"
-        guarded_declaration += f"#define {name_guard}\n"
-        guarded_declaration += f"\n{expression}\n\n#endif\n"
-        return guarded_declaration
-    
-    @staticmethod
-    def declare_expression(expression:str, name:Optional[str]=None):
-        declaration = RooProcDeclare.create_declaration(expression, name)
-        status = ROOT.gInterpreter.Declare(declaration)
-        return status
     
     def _execute(self, processor, **params):
         name = params.get("name", None)
         expression = params['expression']
-        self.declare_expression(expression, name)
+        declare_expression(expression, name)
         return processor
     
 class RooProcSaveFrame(RooProcHelperAction):
@@ -413,6 +431,7 @@ ACTION_MAP = {
     'DECLARE': RooProcDeclare,
     'GLOBAL': RooProcGlobalVariables,
     'ALIAS': RooProcAlias,
+    'SAFEALIAS': RooProcSafeAlias,
     'DEFINE': RooProcDefine,
     'FILTER': RooProcFilter,
     'SAVE': RooProcSave,
