@@ -2,13 +2,14 @@ import quickstats
 
 from .color_schemes import *
 from .abstract_plot import AbstractPlot
-from .multi_data_plot import MultiDataPlot
 from .hypotest_inverter_plot import HypoTestInverterPlot
 from .score_distribution_plot import score_distribution_plot
 from .test_statistic_distribution_plot import TestStatisticDistributionPlot
 from .upper_limit_1D_plot import UpperLimit1DPlot
 from .upper_limit_2D_plot import UpperLimit2DPlot
+from .upper_limit_benchmark_plot import UpperLimitBenchmarkPlot
 from .likelihood_1D_plot import Likelihood1DPlot
+from .pdf_distribution_plot import PdfDistributionPlot
 
 from matplotlib import style, colors
 
