@@ -1,33 +1,44 @@
 from typing import Dict, Optional, Union, List
 import pandas as pd
 
-from quickstats.plots import MultiDataPlot
+from quickstats.plots import AbstractPlot
 from quickstats.plots.template import create_transform
 from quickstats.utils.common_utils import combine_dict
 
-class Likelihood1DPlot(MultiDataPlot):
+class Likelihood1DPlot(AbstractPlot):
     
     CONFIG = {
         'sigma_values': (1, 4),
+        'sigma_pos': 0.93,
         'sigma_names': ('1 $\sigma$', '2 $\sigma$'),
         'sigma_line_styles':{
             'color': 'gray',
             'linestyle': '--'
         }
     }
+    
     def __init__(self, data_map:Union[pd.DataFrame, Dict[str, pd.DataFrame]],
                  label_map:Optional[Dict]=None,
                  styles_map:Optional[Dict]=None,
-                 color_cycler=None,
+                 color_cycle=None,
                  styles:Optional[Union[Dict, str]]=None,
-                 analysis_label_options:Optional[Dict]=None):
-        super().__init__(data_map=data_map,
-                         label_map=label_map,
-                         styles_map=styles_map,
-                         color_cycler=color_cycler,
+                 analysis_label_options:Optional[Dict]=None,
+                 config:Optional[Dict]=None):
+        
+        self.data_map = data_map
+        self.label_map = label_map
+        self.styles_map = styles_map
+        
+        super().__init__(color_cycle=color_cycle,
                          styles=styles,
-                         analysis_label_options=analysis_label_options)
-        self.config = combine_dict(self.CONFIG)
+                         analysis_label_options=analysis_label_options,
+                         config=config)
+        
+    def get_default_legend_order(self):
+        if not isinstance(self.data_map, dict):
+            return []
+        else:
+            return list(self.data_map)
         
     def draw_single_data(self, ax, data:pd.DataFrame, 
                          styles:Optional[Dict]=None,
@@ -42,7 +53,7 @@ class Likelihood1DPlot(MultiDataPlot):
     def draw(self, xattrib:str='mu', yattrib:str='qmu', xlabel:Optional[str]=None, 
              ylabel:Optional[str]="$-2\Delta ln(L)$",
              ymax:float=7, xmin:Optional[float]=None, xmax:Optional[float]=None,
-             draw_sigma_line:bool=True):
+             draw_sigma_line:bool=True, draw_sm_line:bool=False):
         ax = self.draw_frame()
         if isinstance(self.data_map, pd.DataFrame):
             self.draw_single_data(ax, self.data_map, self.styles_map,
@@ -75,17 +86,32 @@ class Likelihood1DPlot(MultiDataPlot):
             sigma_line_styles = self.config['sigma_line_styles']
             sigma_values = self.config['sigma_values']
             sigma_names = self.config['sigma_names']
-            ax.hlines(sigma_values, xmin=0, xmax=1, transform=transform,
+            ax.hlines(sigma_values, xmin=0, xmax=1, zorder=0, transform=transform,
                       **sigma_line_styles)
             if sigma_names:
+                sigma_pos = self.config['sigma_pos']
                 for sigma_value, sigma_name in zip(sigma_values, sigma_names):
-                    ax.text(0.93, sigma_value, sigma_name, color='gray', ha='left', va='bottom', fontsize=20, transform=transform)
+                    ax.text(sigma_pos, sigma_value, sigma_name, color='gray', ha='left',
+                            va='bottom' if (sigma_pos > 0 and sigma_pos < 1) else 'center', fontsize=20, transform=transform)
+            
+        if draw_sm_line:
+            transform = create_transform(transform_y="axis", transform_x="data")
+            sm_line_styles = self.config['sm_line_styles']
+            sm_values = self.config['sm_values']
+            sm_names = self.config['sm_names']
+            ax.vlines(sm_values, ymin=0, ymax=1, zorder=0, transform=transform,
+                      **sm_line_styles)
+            if sm_names:
+                sm_pos = self.config['sm_pos']
+                for sm_value, sm_name in zip(sm_values, sm_names):
+                    ax.text(sm_value, sm_pos, sm_name, color='gray', ha='right', rotation=90,
+                            va='bottom' if (sm_pos > 0 and sm_pos < 1) else 'center', fontsize=20, transform=transform)
             
         handles, labels = self.get_legend_handles_labels()
         ax.legend(handles, labels, **self.styles['legend'])
         ax.set_ylim([0, ymax])
         ax.set_xlim([xmin, xmax])
-        
-        self.draw_axis_labels(ax, xlabel=xlabel, ylabel=ylabel)
+
+        self.draw_axis_components(ax, xlabel=xlabel, ylabel=ylabel)
         
         return ax
