@@ -13,9 +13,9 @@ import matplotlib.transforms as transforms
 from matplotlib.patches import Rectangle
 from matplotlib.backends.backend_pdf import PdfPages
 from quickstats.utils.process_tools import parallel_run
-from quickstats.plots.template import draw_atlas_label, format_axis_ticks, parse_transform, draw_hatches, \
+from quickstats.plots.template import draw_analysis_label, format_axis_ticks, parse_transform, draw_hatches, \
                                       draw_sigma_bands, draw_sigma_lines, get_box_dimension, draw_text, centralize_axis
-from quickstats.components.numerics import ceildiv, pretty_float
+from quickstats.maths.numerics import ceildiv
 from quickstats.utils.common_utils import json_load
 
 BASE_STYLE = {
@@ -471,7 +471,7 @@ class NPRankingPlot:
             draw_options['energy'] = energy
             draw_options['lumi'] = lumi
           
-        draw_atlas_label(**draw_options)
+        draw_analysis_label(**draw_options)
         
     @staticmethod
     def parse_style(style:str):
